package ca.mcgill.cs.swevo.diffcat.view;

import java.net.URL;

import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.Path;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.jface.viewers.LabelProvider;
import org.eclipse.swt.graphics.Image;

import ca.mcgill.cs.swevo.semdiff.core.Result;
import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;

public class CategoryViewLabelProvider extends LabelProvider {

	private final Image ADD = getImageFrom("icons/add.gif");
	private final Image REM = getImageFrom("icons/remove.gif");
	private final Image MOD = getImageFrom("icons/synced.gif");
	
	public String getText(Object obj) {
		
		if (obj instanceof DiffCatNode) {
			return getText( ((DiffCatNode) obj).getResult());
		}
		
		return obj.toString();
	}
	
	public Image getImage(Object obj) {
		
		if (obj instanceof DiffCatResult) {
			return _getImage(obj);
		}
		else if (obj instanceof DiffCatNode) {
			return _getImage (((DiffCatNode) obj).getResult());
		}
		
		return null;
	}
	
	private Image _getImage(Object element) {
		Result result = (Result) element;

		if (result.getHandleFrom() == null && result.getHandleTo() != null) {
			return ADD;
		} else if (result.getHandleFrom() != null && result.getHandleTo() == null) {
			return REM;
		} else {
			return MOD;
		}
		
	}

	private Image getImageFrom (String path) {
		URL url = FileLocator.find(Activator.getDefault().getBundle(), Path.fromOSString(path), null);
		return ImageDescriptor.createFromURL(url).createImage();
	}
	
	private String getText (DiffCatResult result) {
		String changeType = result.getChangeType();
		if (changeType.equals(ChangeOperation.CLASS_INSERT.name())) {
			return ChangeOperation.CLASS_INSERT.name() + " " + result.getRightClassSignature();
		}
		else if (changeType.equals(ChangeOperation.CLASS_DELETE.name())) {
			return ChangeOperation.CLASS_DELETE.name() + " " + result.getLeftClassSignature();
		}
		else if (changeType.equals(ChangeOperation.FUNCTIONALITY_DELETE.name())) {
			return ChangeOperation.FUNCTIONALITY_DELETE.name() + " " + result.getLeftMethodSignature();
		}
		else if (changeType.equals(ChangeOperation.FUNCTIONALITY_INSERT.name())) {
			return ChangeOperation.FUNCTIONALITY_INSERT.name() + " " + result.getRightMethodSignature();
		}
		else if (changeType.equals(ChangeOperation.TRIVIAL_CONSTRUCTOR_DELETE.name())) {
			return ChangeOperation.TRIVIAL_CONSTRUCTOR_DELETE.name() + " " + result.getLeftMethodSignature();
		}
		else if (changeType.equals(ChangeOperation.TRIVIAL_CONSTRUCTOR_INSERT.name())) {
			return ChangeOperation.TRIVIAL_CONSTRUCTOR_INSERT.name() + " " + result.getRightMethodSignature();
		}
		
		return result.toString();
	}
}
