package ca.mcgill.cs.swevo.diffcat.view;

import java.util.Comparator;

import org.eclipse.jface.viewers.Viewer;
import org.eclipse.jface.viewers.ViewerSorter;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.model.DiffCodes;

public class NodeSorter extends ViewerSorter {

	private final DiffCatComparator differ = new DiffCatComparator();
	
	public static class DiffCatComparator implements Comparator<DiffCatResult> {
		@Override
		public int compare(DiffCatResult arg0, DiffCatResult arg1) {
			
			int leftPosFrom = arg0.getPosFrom();
			int rightPosFrom = arg1.getPosFrom();
			
			int leftPosTo = arg0.getPosTo();
			int rightPosTo = arg1.getPosTo();
			
			ChangeOperation op1 = ChangeOperation.valueOf(arg0.getChangeType());
			ChangeOperation op2 = ChangeOperation.valueOf(arg1.getChangeType());
			
			if (op1.isInsert()) {
				if (! op2.isDelete()) {
					return leftPosTo - rightPosTo;
				} else {
					return leftPosTo - rightPosFrom;
				}
			}
			else if (op1.isDelete()) {
				if (! op2.isInsert()) {
					return leftPosFrom - rightPosFrom;
				} else {
					return leftPosFrom - rightPosTo;
				}
			}
			else {
				if (op2.isInsert()) {
					return leftPosTo - rightPosTo;
				}
				else if (op2.isDelete()) {
					return leftPosFrom - rightPosFrom;
				}
				else {
					return leftPosTo - rightPosTo;
				}
			}
		}
	}
	
	public int compare (Object e1, Object e2) {
		if (e1.getClass() == e2.getClass()) {
			if (e1 instanceof ClassSigNode) {
				// classes are sorted alphabetically
				return e1.toString().compareTo(e2.toString());
			}
			else if (e1 instanceof MethodSigNode) {
				// methods are sorted alphabetically
				return e1.toString().compareTo(e2.toString());
			}
			else if (e1 instanceof FieldSigNode) {
				// fields are sorted alphabetically
				return e1.toString().compareTo(e2.toString());
			}
			else if (e1 instanceof DiffCatNode) {
				// this is high- or low-level 
				DiffCatResult r1 = ((DiffCatNode) e1).getResult();
				DiffCatResult r2 = ((DiffCatNode) e2).getResult();
				
				return differ.compare(r1, r2);
			}
			else {
				return 0;	// don't ever expect this
			}
		}
		else {
			if (e1 instanceof FieldSigNode && e2 instanceof MethodSigNode) {
				return -1;
			}
			else if (e1 instanceof MethodSigNode && e2 instanceof FieldSigNode) {
				return +1;
			}
			else if (e1 instanceof DiffCatNode && e2 instanceof FieldSigNode) {
				DiffCatNode node = (DiffCatNode) e1;
				DiffCatResult r = node.getResult();
				
				if (DiffCodes.DOCUMENT_DIFF.passesFilter(r)) {
					return -1;
				}
				
				return +1;	// e1 is a method
			}
			else if (e1 instanceof FieldSigNode && e2 instanceof DiffCatNode) {
				
				DiffCatNode node = (DiffCatNode) e2;
				DiffCatResult r = node.getResult();
				
				if (DiffCodes.DOCUMENT_DIFF.passesFilter(r)) {
					return +1;
				}
				
				return -1;	// e2 is a method
			}
			else if (e1 instanceof DiffCatNode && e2 instanceof MethodSigNode) {
				return -1;	// show functionality insertions/deletions first
			}
			else if (e1 instanceof MethodSigNode && e2 instanceof DiffCatNode) {
				return +1;
			}
		}
		
		return 0;
	}
	
	public int compare(Viewer viewer, Object e1, Object e2) {
		return compare(e1, e2);
	}
	
}
