package ca.mcgill.cs.swevo.diffcat.view.diff;

import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.LinkedList;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.FileLocator;
import org.eclipse.core.runtime.IStatus;
import org.eclipse.core.runtime.Path;
import org.eclipse.core.runtime.Status;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.swt.graphics.Image;

import ca.mcgill.cs.swevo.diffcat.Activator;

public class ThreeChunkFileNode extends FileNode {

	public ThreeChunkFileNode(File file, int pos, int len, String diff) throws CoreException {
		super(file, pos, len, diff);
	}

	@Override
	protected void initByteChunks(int pos, int len, String diff) throws CoreException 
	{
		byte[] left, middle, right;
		
		InputStream is = getContents();
		
		try {
			left = new byte[pos];
			for (int i = 0; i < pos; i++) {
				left[i] = (byte) is.read();
			}
			
			middle = new byte[len];
			for (int i = pos; i < pos + len; i++) {
				middle[i-pos] = (byte) is.read();
			}
			
			LinkedList<Byte> rightB = new LinkedList<Byte>();
			while (true) {
				byte c = (byte) is.read();
				if (c == -1) {
					break;
				}
				rightB.add(c);
			}
			right = new byte[rightB.size()];
			int i = 0;
			for (Byte b : rightB) {
				right[i++] = b;
			}
			
		} catch (IOException e) {
			throw new CoreException(new Status(IStatus.ERROR,"me",e.getMessage(),e));
		}
		
		this.left = new ByteChunk(left, " Top - " + getName(), getType(), getImage("icons/edit.gif"));
		this.middle = new ByteChunk(middle, diff, getType(), getImage("icons/synced.gif"));
		this.right = new ByteChunk(right, " Bottom - " + getName(), getType(), getImage("icons/edit.gif"));
	}

	private Image getImage (String path) {
		URL url = FileLocator.find(Activator.getDefault().getBundle(), Path.fromOSString(path), null);
		return ImageDescriptor.createFromURL(url).createImage();
	}
	
	public Object[] getChildren()
	{
		return new Object[] {left, middle, right};
	}
	
}
