package ca.mcgill.cs.swevo.diffcat.view.filters;

import java.util.HashSet;
import java.util.Set;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.filter.IsNotChangeOperationFilter;
import ca.mcgill.cs.swevo.diffcat.view.Activator;
import ca.mcgill.cs.swevo.diffcat.view.preferences.PreferenceConstants;
import ca.mcgill.cs.swevo.util.filter.IFilter;

/**
 * Utility class for building a chain of filters based on diff preference page 
 * @author dkawry
 *
 */
public class DiffFilterChainBuilder {

	public IFilter<DiffCatResult> makePreferenceFilter () {
		Set<String> ops = new HashSet<String>();
		checkRenameInduced(ops);
		checkTrivialThis(ops);
		checkDocumentDiffs(ops);
		checkVariableRenames(ops);
		checkMethodRenames(ops);
		checkFieldRenames(ops);
		checkParamRenames(ops);
		checkTrivialIf(ops);
		return new IsNotChangeOperationFilter(ops);
	}

	private void checkRenameInduced(Set<String> ops) {
		boolean boolean1 = getBoolean(PreferenceConstants.P_FILTER_SHOW_RENAME_INDUCED);
		if (! boolean1) {
			ops.add(ChangeOperation.AFFECTED_ONLY_BY_RENAMES.name());
		}
	}
	
	private void checkTrivialThis (Set<String> ops) {
		boolean boolean1 = getBoolean(PreferenceConstants.P_FILTER_SHOW_TRIVIAL_THIS);
		if (! boolean1) {
			ops.add(ChangeOperation.TRIVIAL_THIS_EXPRESSION_DELETE.name());
			ops.add(ChangeOperation.TRIVIAL_THIS_EXPRESSION_INSERT.name());
		}
	}
	
	private void checkDocumentDiffs (Set<String> ops) {
		boolean boolean1 = getBoolean(PreferenceConstants.P_FILTER_SHOW_DOCUMENT_DIFFS);
		if (! boolean1) {
			ops.add(ChangeOperation.DOC_DELETE.name());
			ops.add(ChangeOperation.DOC_INSERT.name());
			ops.add(ChangeOperation.DOC_UPDATE.name());
		}
	}
	
	private void checkVariableRenames(Set<String> ops) {
		boolean boolean1 = getBoolean(PreferenceConstants.P_FILTER_SHOW_VARIABLE_RENAMES);
		if (! boolean1) {
			ops.add(ChangeOperation.VARIABLE_NAME_UPDATE.name());
		}
	}
	
	private void checkParamRenames (Set<String> ops) {
		boolean boolean1 = getBoolean(PreferenceConstants.P_FILTER_SHOW_PARAM_RENAMES);
		if (! boolean1) {
			ops.add(ChangeOperation.PARAMETER_NAME_UPDATE.name());
		}
	}
	
	private void checkFieldRenames (Set<String> ops) {
		boolean boolean1 = getBoolean(PreferenceConstants.P_FILTER_SHOW_FIELD_RENAMES);
		if (! boolean1) {
			ops.add(ChangeOperation.ATTRIBUTE_NAME_UPDATE.name());
		}
	}
	
	private void checkMethodRenames (Set<String> ops) {
		boolean boolean1 = getBoolean(PreferenceConstants.P_FILTER_SHOW_METHOD_RENAMES);
		if (! boolean1) {
			ops.add(ChangeOperation.METHOD_NAME_UPDATE.name());
		}
	}
	
	private void checkTrivialIf (Set<String> ops) {
		boolean boolean1 = getBoolean(PreferenceConstants.P_FILTER_SHOW_TRIVIAL_IF);
		if (! boolean1) {
			ops.add(ChangeOperation.TRIVIAL_IF_STATEMENT_UPDATE.name());
		}
	}
	
	public boolean isFilterProperty (String property) {
		return 
		 property.equals(PreferenceConstants.P_FILTER_SHOW_RENAME_INDUCED) || 
		 property.equals(PreferenceConstants.P_FILTER_SHOW_DOCUMENT_DIFFS) || 
		 property.equals(PreferenceConstants.P_FILTER_SHOW_FIELD_RENAMES) || 
		 property.equals(PreferenceConstants.P_FILTER_SHOW_METHOD_RENAMES) || 
		 property.equals(PreferenceConstants.P_FILTER_SHOW_PARAM_RENAMES) || 
		 property.equals(PreferenceConstants.P_FILTER_SHOW_TRIVIAL_IF) || 
		 property.equals(PreferenceConstants.P_FILTER_SHOW_TRIVIAL_THIS) || 
		 property.equals(PreferenceConstants.P_FILTER_SHOW_VARIABLE_RENAMES);
	}
	
	private boolean getBoolean (String pref) {
		return Activator.getDefault().getPreferenceStore().getBoolean(pref);
	}
	
}
