/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.analysis;

import java.util.HashSet;
import java.util.Set;

import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.util.CollectionMap;
import ca.mcgill.cs.swevo.util.CollectionUtil;

public abstract class AbstractSimilarityScorer implements ISimilarityScorer {

	private double MIN_THRESH;
	
	public void setMinThresh (double minThresh) {
		MIN_THRESH = minThresh;
	}
	
	@Override
	public void trimCandidateMap(CollectionMap<DiffCatResult, DiffCatResult> candidates, Set<DiffCatResult> knowns) {
		
		Set<DiffCatResult> inserts = new HashSet<DiffCatResult>(candidates.values());
		
		for (DiffCatResult insert : inserts) {
			double maxOverlap = MIN_THRESH;
			Set<DiffCatResult> bestDeletes = new HashSet<DiffCatResult>();
			for (DiffCatResult delete : candidates.keySet()) {
				double overlap = getSimilarityScore (delete, insert, knowns);
				if (overlap > maxOverlap) {
					bestDeletes.clear();
					maxOverlap = overlap;
				}
				if (overlap == maxOverlap) {
					bestDeletes.add(delete);
				}
			}
			
			candidates.removeValue(insert);
			
			if (bestDeletes.size() == 1) {
				// exactly one delete to match this insert 
				// we predict it's the right one and set
				// the delete accordingly
				
				DiffCatResult bestDelete = CollectionUtil.getFirst(bestDeletes);
				candidates.removeKey(bestDelete);
				candidates.add(bestDelete, insert);
			}
			
			else if (bestDeletes.size() > 1) {
				
				// use textual similarity of methods for
				// tiebreaker TODO
				
			}
		}
		
		trimEmpties(candidates);
	}

	void trimEmpties(CollectionMap<DiffCatResult, DiffCatResult> candidates) {
		// trim empties
		for (DiffCatResult delete : new HashSet<DiffCatResult>(candidates.keySet())) {
			if (candidates.get(delete).isEmpty()) {
				candidates.removeKey(delete);
			}
		}
	}
	
}
