/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.analysis;

import org.eclipse.core.runtime.CoreException;
import org.eclipse.core.runtime.IConfigurationElement;
import org.eclipse.core.runtime.Platform;

public class ClassRenameDetector {

	private static final String ICLASSRENAMEDETECTOR_ID = "ca.mcgill.cs.swevo.diffcat.classRenameDetector";

	private ISimilarityScorer renameDetector;
	
	public ClassRenameDetector () {
		loadClassRenameDetector();
	}
	
	private void loadClassRenameDetector() {
		IConfigurationElement[] config = Platform.getExtensionRegistry()
				.getConfigurationElementsFor(ICLASSRENAMEDETECTOR_ID);
		try {
			
			for (IConfigurationElement e : config) {
				final Object o = e.createExecutableExtension("class");
				if (o instanceof ISimilarityScorer) {
					renameDetector = (ISimilarityScorer) o;
				}
			}
			
		} catch (CoreException ex) {
			System.out.println(ex.getMessage());		// TODO
		}
		
		if (renameDetector == null) {
			renameDetector = new DefaultClassRenameDetector();
		}
	}
	
	public ISimilarityScorer getScorer () {
		return renameDetector;
	}
}
