/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.cd.finder;

import java.util.Arrays;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.AnonymousClassDeclaration;
import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.Type;

import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;


public abstract class ANodeFinder extends ASTVisitor {

	protected ASTNode assocNode;

	public ASTNode getAssociatedNode() {
		return assocNode;
	}

	public boolean visit (AnonymousClassDeclaration declaration) {
		return false;
	}
	
	public boolean foundAssociatedNode() {
		return assocNode != null;
	}
	
	protected boolean fullyOrPartialMatches (String[] fqn1, String[] fqn2) {
		return fullyMatches (fqn1, fqn2) || partialMatches(fqn1, fqn2);
	}
	
	protected boolean fullyMatches (String[] fqn1, String[] fqn2) {
		return Arrays.equals(fqn1, fqn2);
	}
	
	protected boolean partialMatches (String[] fqn1, String[] fqn2) {
		
		int last1 = fqn1.length - 1;
		int last2 = fqn2.length - 1;
		
		return fqn1[last1].equals(fqn2[last2]);
	}
	
	protected boolean partialMatches (String[][] fqns1, String[][] fqns2) {
		if (fqns2.length != fqns1.length) {
			return false;
		}
		
		for (int i = 0; i < fqns1.length; i++)
		{
			if (! fullyMatches(fqns1[i], fqns2[i]) && 
				 ! partialMatches(fqns1[i], fqns2[i]))
			{
				return false;
			}
		}
		
		return true;
	}
	
	protected boolean inCorrectMethodDecl (ASTNode node, ASTNode methodDecl) {
		ASTNode parent = node.getParent(); 
		while (parent != null && parent != methodDecl) {
			parent = parent.getParent();
		}
		
		return parent == methodDecl;
	}
	
	protected String[] getType (FieldDeclaration decl) {
		Type type = decl.getType();
		if (type.resolveBinding() != null) {
			ITypeBinding binding = type.resolveBinding();
			if (binding.getQualifiedName() != null) {
				return BindingUtil.getTypeFQN(binding);
			}
			if (binding.getName() != null) {
				return new String[] {binding.getName()};
			}
			
			return BindingUtil.UNKNOWN_TYPE_BINDING;
		}
		else {
			String typeString = type.toString();
			return typeString.split("\\.");
		}
	}
}
