/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.cd.finder;

import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.evolizer.changedistiller.model.entities.SourceCodeEntity;
import org.evolizer.changedistiller.model.entities.StructureEntityVersion;

import ca.mcgill.cs.swevo.diffcat.util.ast.ASTUtil;
import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;


public class MethodDeclarationFinder extends ANodeFinder {

	private String   declName;
	private String[][] declTypeArgs;
	private String[] declType;
	
	List<MethodDeclaration> secondaries = new LinkedList<MethodDeclaration>();
	
	/**
	 * Initialize a new MethodDeclarationFinder
	 * @param methodDecl METHOD_DECLARATION of the form fqn.name(arg1,...,arg2)
	 */
	public MethodDeclarationFinder (SourceCodeEntity methodDecl)
	{
		this.declName 		= CDUtil.getMethodName(methodDecl);
		this.declTypeArgs	= CDUtil.getMethodParameterTypes(methodDecl);
		this.declType 		= CDUtil.getMethodDeclaringType(methodDecl);
	}
	
	/**
	 * Initialize a new MethodDeclarationFinder
	 * @param methodDecl METHOD_DECLARATION of the form fqn.name(arg1,...,arg2)
	 */
	public MethodDeclarationFinder (StructureEntityVersion methodDecl)
	{
		this.declName 		= CDUtil.getMethodName(methodDecl);
		this.declTypeArgs	= CDUtil.getMethodParameterTypes(methodDecl);
		this.declType 		= CDUtil.getMethodDeclaringType(methodDecl);
	}
		
	public boolean visit (MethodDeclaration node) {
		String 	 declName = null;
		String[][] declTypeArgs = null;
		
		declName = ASTUtil.getMethodName(node);
		
		if (! declName.equals(this.declName)) {
			return true;
		}
	
		declTypeArgs = ASTUtil.getParamTypesViaBindings(node);
		if (super.partialMatches(this.declTypeArgs, declTypeArgs)) {
			return setAssocNode(node);
		}
		
		// could also have a rename refactoring of a type affecting the args
		declTypeArgs = ASTUtil.getParamTypesViaStrings(node);
		if (super.partialMatches(this.declTypeArgs, declTypeArgs)) {
			return setAssocNode(node);
		}
		
		return true;
	}
	
	private boolean setAssocNode(MethodDeclaration node) {
		if (super.assocNode == null) {
			super.assocNode = node;
			return false;
		}
		else {
			String declType[] = BindingUtil.getDeclaringType(((MethodDeclaration) super.assocNode).resolveBinding());
			if (super.fullyMatches(declType, this.declType)) {
				return false;
			}
			
			String declType2[] = BindingUtil.getDeclaringType(node.resolveBinding());
			if (super.fullyMatches(declType2, this.declType)) {
				super.assocNode = node;
				return false;
			}
			
			if (! super.fullyOrPartialMatches(declType, this.declType) && 
				super.fullyOrPartialMatches(declType2, this.declType)) {
				super.assocNode = node;
				return false;
			}
			
			if (! super.fullyOrPartialMatches(declType2, this.declType)) {
				return false;
			}
			
			// otherwise can only store as secondary (figure out if we need to)
			secondaries.add(node);
			return false;
		}
	}
	
}
