/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.cd.finder;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ReturnStatement;
import org.eclipse.jdt.core.dom.Statement;
import org.evolizer.changedistiller.model.entities.SourceCodeEntity;

public class ReturnStatementFinder extends ANodeFinder {

	private final ASTNode methodDecl;
	private final String statement;
	private boolean stopVisiting;
	
	public ReturnStatementFinder (SourceCodeEntity statement, ASTNode methodDecl) {
		this.methodDecl = methodDecl;
		this.statement = statement.getUniqueName();
		this.stopVisiting = false;
	}
	
	private boolean visitStatementExpr (Statement statement, Expression expression)
	{
		if (stopVisiting) {
			return false;
		}
		
		if (! super.inCorrectMethodDecl(statement, methodDecl)) {
			return true;
		}
		
		// e.g. return;
		if (expression == null) {
			if (this.statement.isEmpty()) {
				super.assocNode = statement;
				stopVisiting = true;
			}
			return true;
		}
		
		// e.g. return;
		if (expression.toString() == null) {
			return true;
		}
		
		String statementString = expression.toString().trim();
		if (statementString.equals(this.statement)) {
			super.assocNode = statement;
			this.stopVisiting = true;
		}
		
		return true;
	}
	
	public boolean visit(ReturnStatement s) {
		return visitStatementExpr(s, s.getExpression());
	}
}
