/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.cd.finder;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.AnonymousClassDeclaration;
import org.eclipse.jdt.core.dom.AssertStatement;
import org.eclipse.jdt.core.dom.Assignment;
import org.eclipse.jdt.core.dom.Block;
import org.eclipse.jdt.core.dom.ConstructorInvocation;
import org.eclipse.jdt.core.dom.DoStatement;
import org.eclipse.jdt.core.dom.EmptyStatement;
import org.eclipse.jdt.core.dom.EnhancedForStatement;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ExpressionStatement;
import org.eclipse.jdt.core.dom.IfStatement;
import org.eclipse.jdt.core.dom.LabeledStatement;
import org.eclipse.jdt.core.dom.Statement;
import org.eclipse.jdt.core.dom.SuperConstructorInvocation;
import org.eclipse.jdt.core.dom.SwitchCase;
import org.eclipse.jdt.core.dom.SwitchStatement;
import org.eclipse.jdt.core.dom.SynchronizedStatement;
import org.eclipse.jdt.core.dom.ThrowStatement;
import org.eclipse.jdt.core.dom.TypeDeclarationStatement;
import org.eclipse.jdt.core.dom.VariableDeclarationStatement;
import org.eclipse.jdt.core.dom.WhileStatement;
import org.evolizer.changedistiller.model.entities.SourceCodeEntity;

public class StatementFinder extends ANodeFinder {

	private final ASTNode methodDecl;
	private final String statement;
	private boolean stopVisiting;
	
	public StatementFinder (SourceCodeEntity statement, ASTNode methodDecl) {
		this.methodDecl = methodDecl;
		this.statement = statement.getUniqueName();
		this.stopVisiting = false;
	}
	
	private boolean visitRaw (Statement statement) {
		if (stopVisiting) {
			return false;
		}
		
		if (! super.inCorrectMethodDecl(statement, methodDecl)) {
			return true;
		}
		
		String statementString = statement.toString().trim();
		if (! statementString.equals(this.statement)) {
			return true;
		}
		
		super.assocNode = statement;
		this.stopVisiting = true;
		
		return false;
	}
	
	private boolean visitRaw (Assignment statement) {
		if (stopVisiting) {
			return false;
		}
		
		if (! super.inCorrectMethodDecl(statement, methodDecl)) {
			return true;
		}
		
		String statementString = statement.toString().trim();
		if (! statementString.equals(this.statement)) {
			return true;
		}
		
		super.assocNode = statement;
		this.stopVisiting = true;
		
		return false;
	}
	
	private boolean visitStatementExpr (Statement statement, Expression expression)
	{
		if (stopVisiting) {
			return false;
		}
		
		if (! super.inCorrectMethodDecl(statement, methodDecl)) {
			return true;
		}
		
		// e.g. return;
		if (expression == null) {
			return true;
		}
		
		// e.g. return;
		if (expression.toString() == null) {
			return true;
		}
		
		String statementString = expression.toString().trim();
		if (statementString.equals(this.statement)) {
			super.assocNode = expression;
			this.stopVisiting = true;
		}
		
		return true;
	}
	
	public boolean visit (AnonymousClassDeclaration declaration) {
		return true;
	}
	public boolean visit (Assignment s) {
		return visitRaw (s);
	}
	public boolean visit(Block s) {
		return visitRaw (s);
	}
	public boolean visit(EnhancedForStatement s) {
		return visitRaw (s);
	}
	public boolean visit(WhileStatement s) {
		return visitStatementExpr (s, s.getExpression()); // this is just how CD does it...
	}
	public boolean visit (IfStatement s) {
		return visitStatementExpr (s, s.getExpression()); // this is just how CD does it...
	}
	public boolean visit(DoStatement s) {
		return visitRaw (s);
	}
	public boolean visit(SwitchStatement s) {
		return visitStatementExpr (s, s.getExpression());
	}
	public boolean visit(SwitchCase s) {
		return visitStatementExpr (s, s.getExpression());
	}
	public boolean visit(SynchronizedStatement s) {
		return visitStatementExpr (s, s.getExpression());
	}
	public boolean visit(ThrowStatement s) {
		return visitStatementExpr (s, s.getExpression());
	}
	public boolean visit(EmptyStatement s) {
		return visitRaw (s);
	}
	public boolean visit(ExpressionStatement s) {
		return visitRaw (s);
	}
	public boolean visit(LabeledStatement s) {
		return visitRaw (s);
	}
	public boolean visit(AssertStatement s) {
		return visitRaw (s);
	}
	public boolean visit(VariableDeclarationStatement s) {
		return visitRaw (s);
	}
	public boolean visit(TypeDeclarationStatement s) {
		return visitRaw (s);
	}
	public boolean visit(ConstructorInvocation s) {
		return visitRaw (s);
	}
	public boolean visit(SuperConstructorInvocation s) {
		return visitRaw (s);
	}
}

