/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.cd.finder;

import org.eclipse.jdt.core.dom.AbstractTypeDeclaration;
import org.eclipse.jdt.core.dom.EnumDeclaration;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.TypeDeclaration;
import org.evolizer.changedistiller.model.entities.SourceCodeEntity;
import org.evolizer.changedistiller.model.entities.StructureEntityVersion;

public class TypeDeclarationFinder extends ANodeFinder {

	private String[] fqn;
	private boolean foundFullMatch = false;
	
	public TypeDeclarationFinder (SourceCodeEntity element) {
		this.fqn = CDUtil.getTypeFQN(element);
	}
	
	public TypeDeclarationFinder (StructureEntityVersion element) {
		this.fqn = CDUtil.getTypeFQN(element);
	}
	
	public boolean visit (TypeDeclaration node) {
		return visit ((AbstractTypeDeclaration) node);
	}
	
	public boolean visit (EnumDeclaration node) {
		return visit ((AbstractTypeDeclaration) node);
	}
	
	private boolean visit (AbstractTypeDeclaration node) {
		if (node == null || foundFullMatch) {
			return false;		
		}
		
		ITypeBinding binding = node.resolveBinding();
		if(binding != null) {
			String[] qualBinding = binding.getQualifiedName().split("\\.");
			
			if (super.fullyMatches(this.fqn, qualBinding)) {
				super.assocNode = node;
				foundFullMatch = true;
				return false;
			}
		}
		
		String name = node.getName().toString();
		if (super.fullyOrPartialMatches(this.fqn, new String[]{name})) {
			super.assocNode = node;
			return false;
		}
		
		return true;
	}
}
