/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.cd.finder;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.TryStatement;
import org.eclipse.jdt.core.dom.WhileStatement;
import org.evolizer.changedistiller.model.classifiers.SourceRange;
import org.evolizer.changedistiller.model.entities.SourceCodeEntity;

public class WhileStatementFinder extends ANodeFinder {

	private final ASTNode methodDecl;
	private final SourceRange range;
	private boolean stopVisiting;
	
	public WhileStatementFinder (SourceCodeEntity statement, ASTNode methodDecl) {
		this.methodDecl = methodDecl;
		this.range = statement.getSourceRange();
		this.stopVisiting = false;
	}
	
	public boolean visit (WhileStatement statement) {
		
		// keep the first one, in case we don't find anything (so not null, at least)
		if (stopVisiting) {
			return false;
		}
		
		if (! super.inCorrectMethodDecl(statement, this.methodDecl)) {
			return true;
		}
		
		if (super.assocNode == null) {
			super.assocNode = statement;
		}
		
		int startPosition = statement.getStartPosition();
		int length = statement.getLength();
		if (range.getOffset() <= startPosition && 
			range.getOffset() + range.getLength() >= startPosition + length)
		{
			this.stopVisiting = true;
			super.assocNode = statement;
			return false;
		}
		
		return true;
	} 

}
