/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.model;

import org.evolizer.changedistiller.model.classifiers.ChangeType;
import org.evolizer.changedistiller.model.classifiers.EntityType;

public enum ChangeOperation {

	AFFECTED_BY_VAR_EXTRACTION,
	AFFECTED_ONLY_BY_RENAMES,
	ALTERNATIVE_PART_DELETE,
    ALTERNATIVE_PART_INSERT,
    ASSIGNMENT_INSERT,
    ASSIGNMENT_DELETE,
    ASSIGNMENT_ORDERING_UPDATE,
    ASSIGNMENT_MOVE,
    ASSIGNMENT_UPDATE,
    ASSERT_STATEMENT_DELETE,
    ASSERT_STATEMENT_INSERT,
    ASSERT_STATEMENT_UPDATE,
    ASSERT_STATEMENT_MOVE,
    ASSERT_STATEMENT_ORDERING_UPDATE,
    ATTRIBUTE_ACCESSIBILITY_INSERT,
    ATTRIBUTE_ACCESSIBILITY_DELETE,
    ATTRIBUTE_ACCESSIBILITY_UPDATE,
    ATTRIBUTE_SIGNATURE_UPDATE,
	ATTRIBUTE_MODIFIABILITY_INSERT,
	ATTRIBUTE_MODIFIABILITY_DELETE,
	ATTRIBUTE_NAME_UPDATE, 
    ATTRIBUTE_TYPE_UPDATE,
    ATTRIBUTE_TYPE_RENAME_UPDATE,
    ATTRIBUTE_TYPE_TRIVIAL_UPDATE,
    ATTRIBUTE_INSERT,
    ATTRIBUTE_DELETE,
    ATTRIBUTE_INIT_INSERT,
    ATTRIBUTE_INIT_DELETE,
    ATTRIBUTE_INIT_UPDATE,
    BREAK_STATEMENT_INSERT,
    BREAK_STATEMENT_DELETE,
    BREAK_STATEMENT_UPDATE,
    BREAK_STATEMENT_MOVE,
    BREAK_STATEMENT_ORDERING_UPDATE,
    CATCH_CLAUSE_INSERT,
    CATCH_CLAUSE_DELETE,
    CATCH_CLAUSE_ORDERING_UPDATE,
    CATCH_CLAUSE_MOVE,
    CATCH_CLAUSE_UPDATE,
    CLASS_CAST_INSERT,
    CLASS_CAST_DELETE,
    CLASS_DELETE,
    CLASS_DERIVABILITY_INSERT,
    CLASS_DERIVABILITY_DELETE,
    CLASS_INSERT,
    CLASS_INSTANCE_CREATION_DELETE,
    CLASS_INSTANCE_CREATION_INSERT,
    CLASS_INSTANCE_CREATION_UPDATE,
    CLASS_INSTANCE_CREATION_ORDERING_UPDATE,
    CLASS_INSTANCE_CREATION_MOVE,
    CLASS_NAME_UPDATE,
    COMMENT_DELETE,
    COMMENT_INSERT,
    COMMENT_MOVE,
    COMMENT_UPDATE,
    CONTINUE_STATEMENT_INSERT,
    CONTINUE_STATEMENT_DELETE,
    CONTINUE_STATEMENT_ORDERING_UPDATE,
    CONTINUE_STATEMENT_MOVE,
    CONTINUE_STATEMENT_UPDATE,
    CONDITION_EXPRESSION_UPDATE,
    CONSTRUCTOR_INVOCATION_INSERT,
    CONSTRUCTOR_INVOCATION_DELETE,
    CONSTRUCTOR_INVOCATION_ORDERING_UPDATE,
    CONSTRUCTOR_INVOCATION_MOVE,
    CONSTRUCTOR_INVOCATION_UPDATE,
    DO_STATEMENT_INSERT,
    DO_STATEMENT_DELETE,
    DO_STATEMENT_UPDATE,
    DO_STATEMENT_MOVE,
    DO_STATEMENT_ORDERING_UPDATE,
    DOC_DELETE,
    DOC_INSERT,
    DOC_UPDATE,
    ENHANCED_FOR_STATEMENT_INSERT,
    ENHANCED_FOR_STATEMENT_UPDATE,
    ENHANCED_FOR_STATEMENT_DELETE,
    ENHANCED_FOR_STATEMENT_MOVE,
    ENHANCED_FOR_STATEMENT_ORDERING_UPDATE,
    ENUM_DECLARATION_INSERT,
    ENUM_DECLARATION_DELETE,
    ENUM_CONSTANT_INSERT,
    ENUM_CONSTANT_DELETE,
    FOR_STATEMENT_INSERT,
    FOR_STATEMENT_DELETE,
    FOR_STATEMENT_ORDERING_UPDATE,
    FOR_STATEMENT_MOVE,
    FOR_STATEMENT_UPDATE,
    FUNCTIONALITY_INSERT,
    FUNCTIONALITY_DELETE,
    IF_STATEMENT_INSERT,
    IF_STATEMENT_DELETE,
    IF_STATEMENT_ORDERING_UPDATE,
    IF_STATEMENT_MOVE,
    IF_STATEMENT_UPDATE,
    LABELED_STATEMENT_DELETE,
    LABELED_STATEMENT_INSERT,
    LABELED_STATEMENT_UPDATE,
    LABELED_STATEMENT_MOVE,
    LABELED_STATEMENT_ORDERING_UPDATE,
    LOCAL_VARIABLE_EXTRACTION_INSERT,
    METHOD_SIGNATURE_UPDATE,
    METHOD_INVOCATION_INSERT,
    METHOD_INVOCATION_DELETE,
    METHOD_INVOCATION_ORDERING_UPDATE,
    METHOD_INVOCATION_MOVE,
    METHOD_INVOCATION_UPDATE,
    METHOD_MOVE,
    METHOD_NAME_UPDATE,
    METHOD_OVERRIDABILITY_MODIFIER_INSERT,
    METHOD_OVERRIDABILITY_MODIFIER_DELETE,
    METHOD_OVERRIDABILITY_MODIFIER_UPDATE,
    METHOD_ACCESSIBILITY_INSERT,
    METHOD_ACCESSIBILITY_DELETE,
    METHOD_ACCESSIBILITY_UPDATE,
    PARAMETER_DELETE, 
    PARAMETER_INSERT, 
    PARAMETER_ORDERING_UPDATE, 
    PARAMETER_NAME_UPDATE,
    PARAMETER_TYPE_UPDATE,
    PARAMETER_TYPE_RENAME_UPDATE,
    PARAMETER_TYPE_TRIVIAL_UPDATE,
    PARENT_CLASS_UPDATE,
    PARENT_CLASS_DELETE,
    PARENT_CLASS_INSERT,
    PARENT_CLASS_TRIVIAL_UPDATE,
    PARENT_CLASS_RENAME_UPDATE,
    PARENT_INTERFACE_UPDATE,
    PARENT_INTERFACE_DELETE,
    PARENT_INTERFACE_INSERT,
    PARENT_INTERFACE_TRIVIAL_UPDATE,
    PARENT_INTERFACE_RENAME_UPDATE,
    REDUNDANT_CLASS_CAST_UPDATE,
    RETURN_STATEMENT_INSERT,
    RETURN_STATEMENT_DELETE,
    RETURN_STATEMENT_MOVE,
    RETURN_STATEMENT_UPDATE,
    RETURN_STATEMENT_ORDERING_UPDATE,
    RETURN_TYPE_UPDATE,
    RETURN_TYPE_TRIVIAL_UPDATE,
    RETURN_TYPE_DELETE, 
    RETURN_TYPE_INSERT,
    RETURN_TYPE_RENAME_UPDATE,
    STATEMENT_DELETE,
    STATEMENT_INSERT,
    STATEMENT_ORDERING_UPDATE,
    STATEMENT_MOVE,
    STATEMENT_UPDATE,
    SUPER_CONSTRUCTOR_INVOCATION_INSERT,
    SUPER_CONSTRUCTOR_INVOCATION_DELETE,
    SUPER_CONSTRUCTOR_INVOCATION_ORDERING_UPDATE,
    SUPER_CONSTRUCTOR_INVOCATION_MOVE,
    SUPER_CONSTRUCTOR_INVOCATION_UPDATE,
    SUPER_METHOD_INVOCATION_INSERT,
    SUPER_METHOD_INVOCATION_DELETE,
    SUPER_METHOD_INVOCATION_ORDERING_UPDATE,
    SUPER_METHOD_INVOCATION_MOVE,
    SUPER_METHOD_INVOCATION_UPDATE,
    STRING_ARG_SPLIT,
    SWITCH_CASE_INSERT,
    SWITCH_CASE_DELETE,
    SWITCH_CASE_UPDATE,
    SWITCH_CASE_MOVE,
    SWITCH_CASE_ORDERING_UPDATE,
    SWITCH_STATEMENT_INSERT,
    SWITCH_STATEMENT_DELETE,
    SWITCH_STATEMENT_ORDERING_UPDATE,
    SWITCH_STATEMENT_MOVE,
    SWITCH_STATEMENT_UPDATE,
    SYNCHRONIZED_STATEMENT_INSERT,
    SYNCHRONIZED_STATEMENT_DELETE,
    SYNCHRONIZED_STATEMENT_ORDERING_UPDATE,
    SYNCHRONIZED_STATEMENT_MOVE,
    SYNCHRONIZED_STATEMENT_UPDATE,
    THIS_EXPRESSION_DELETE,
    THIS_EXPRESSION_INSERT,
    THROW_STATEMENT_INSERT,
    THROW_STATEMENT_DELETE,
    THROW_STATEMENT_ORDERING_UPDATE,
    THROW_STATEMENT_MOVE,
    THROW_STATEMENT_UPDATE,
    THROWN_EXCEPTION_INSERT,
    THROWN_EXCEPTION_DELETE,
    THROWN_EXCEPTION_TRIVIAL_UPDATE,
    THROWN_EXCEPTION_RENAME_UPDATE,
    TRIVIAL_IF_STATEMENT_UPDATE,
    TRIVIAL_THIS_EXPRESSION_DELETE,
    TRIVIAL_THIS_EXPRESSION_INSERT,
    TRIVIAL_THIS_EXPRESSION_UPDATE,
    TRIVIAL_METHOD_INVOCATION_DELETE,
    TRIVIAL_METHOD_INVOCATION_INSERT,
    TRIVIAL_RETURN_STATEMENT_DELETE,
    TRIVIAL_RETURN_STATEMENT_INSERT,
    TRIVIAL_CONSTRUCTOR_INSERT,
    TRIVIAL_CONSTRUCTOR_DELETE,
    TRIVIAL_ATTRIBUTE_INIT_DELETE,
    TRIVIAL_VARIABLE_INITIALIZE_DELETE,
    TRIVIAL_ATTRIBUTE_INIT_INSERT,
    TRIVIAL_VARIABLE_INITIALIZE_INSERT,
    TRY_STATEMENT_INSERT,
    TRY_STATEMENT_DELETE,
    TRY_STATEMENT_ORDERING_UPDATE,
    TRY_STATEMENT_MOVE,
    TRY_STATEMENT_UPDATE,
    TYPE_ACCESSIBILITY_INSERT,
    TYPE_ACCESSIBILITY_DELETE,
    TYPE_ACCESSIBILITY_UPDATE,
    UNCLASSIFIED,
    VARIABLE_DECLARATION_STATEMENT_INSERT,
    VARIABLE_DECLARATION_STATEMENT_DELETE,
    VARIABLE_DECLARATION_STATEMENT_ORDERING_UPDATE,
    VARIABLE_DECLARATION_STATEMENT_MOVE,
    VARIABLE_DECLARATION_STATEMENT_UPDATE,
    VARIABLE_DECLARED_TYPE_UPDATE,
    VARIABLE_DECLARED_TYPE_TRIVIAL_UPDATE,
    VARIABLE_DECLARED_TYPE_RENAME_UPDATE,
    VARIABLE_INITIALIZE_INSERT,
    VARIABLE_INITIALIZE_DELETE,
    VARIABLE_INITIALIZE_UPDATE,
    VARIABLE_MODIFIABILITY_DELETE,
    VARIABLE_MODIFIABILITY_INSERT,
    VARIABLE_NAME_UPDATE,
    WHILE_STATEMENT_INSERT,
    WHILE_STATEMENT_DELETE,
    WHILE_STATEMENT_ORDERING_UPDATE,
    WHILE_STATEMENT_MOVE,
    WHILE_STATEMENT_UPDATE;				
	
	public static ChangeOperation mapChangeOperation (ChangeType type) {
		return values()[type.ordinal()];
	}
	
	public static ChangeOperation getOperation (ChangeOp op, ChangeOpEntity entity) {
		return valueOf(entity.name() + "_" + op.name());
	}
	
	public static ChangeOperation getOperation (ChangeOp op, EntityType entity) {
		return valueOf(entity.name() + "_" + op.name());
	}
	
	public static boolean isStatementUpdate (String changeType) {
		ChangeOperation op = ChangeOperation.valueOf(changeType);
		if (op == null) {
			return false;
		}
		
		return op.isStatementUpdate();
	}
	
	public static boolean isStatementInsert (String changeType) {
		ChangeOperation op = ChangeOperation.valueOf(changeType);
		if (op == null) {
			return false;
		}
		return op.isStatementInsert();
	}
	
	public static boolean isStatementDelete (String changeType) {
		ChangeOperation op = ChangeOperation.valueOf(changeType);
		if (op == null) {
			return false;
		}
		return op.isStatementDelete();
	}
	
	public boolean isInsert () {
		return name().endsWith("_INSERT");
	}
	
	public boolean isDelete () {
		return name().endsWith("_DELETE");
	}
	
	public boolean isUpdateBased () {
		return ! isDelete () && ! isInsert ();
	}
	
	private boolean isStatementUpdate () {
		return
			this == ASSERT_STATEMENT_UPDATE ||
			this == ASSIGNMENT_UPDATE ||
			this == BREAK_STATEMENT_UPDATE ||
			this == CATCH_CLAUSE_UPDATE ||
			this == CONDITION_EXPRESSION_UPDATE ||
			this == CONSTRUCTOR_INVOCATION_UPDATE ||
			this == DO_STATEMENT_UPDATE ||
			this == FOR_STATEMENT_UPDATE ||
			this == ENHANCED_FOR_STATEMENT_UPDATE || 
			this == IF_STATEMENT_UPDATE ||
			this == METHOD_INVOCATION_UPDATE ||
			this == RETURN_STATEMENT_UPDATE ||
			this == STATEMENT_UPDATE ||
			this == STRING_ARG_SPLIT || 
			this == SUPER_CONSTRUCTOR_INVOCATION_UPDATE ||
			this == SUPER_METHOD_INVOCATION_UPDATE ||
			this == SWITCH_CASE_UPDATE ||
			this == SWITCH_STATEMENT_UPDATE ||
			this == SYNCHRONIZED_STATEMENT_UPDATE ||
			this == THROW_STATEMENT_UPDATE || 
			this == TRY_STATEMENT_UPDATE  ||
			this == VARIABLE_DECLARATION_STATEMENT_UPDATE ||
			this == VARIABLE_INITIALIZE_UPDATE ||
			this == VARIABLE_DECLARED_TYPE_UPDATE || 
			this == WHILE_STATEMENT_UPDATE ||
			this == AFFECTED_BY_VAR_EXTRACTION; 
	}
	
	public boolean isStatementInsert () {
		return
		this == ASSERT_STATEMENT_INSERT ||
		this == ASSIGNMENT_INSERT ||
		this == BREAK_STATEMENT_INSERT ||
		this == CATCH_CLAUSE_INSERT ||
		this == CONSTRUCTOR_INVOCATION_INSERT ||
		this == DO_STATEMENT_INSERT ||
		this == FOR_STATEMENT_INSERT ||
		this == IF_STATEMENT_INSERT ||
		this == METHOD_INVOCATION_INSERT ||
		this == RETURN_STATEMENT_INSERT ||
		this == STATEMENT_INSERT ||
		this == SUPER_CONSTRUCTOR_INVOCATION_INSERT ||
		this == SUPER_METHOD_INVOCATION_INSERT ||
		this == SWITCH_CASE_INSERT ||
		this == SWITCH_STATEMENT_INSERT ||
		this == SYNCHRONIZED_STATEMENT_INSERT ||
		this == THROW_STATEMENT_INSERT ||
		this == TRY_STATEMENT_INSERT  ||
		this == VARIABLE_DECLARATION_STATEMENT_INSERT ||
		this == VARIABLE_INITIALIZE_INSERT || 
		this == WHILE_STATEMENT_INSERT || 
		this == LOCAL_VARIABLE_EXTRACTION_INSERT ; 
	}
	
	public boolean isStatementDelete () {
		return
		this == ASSERT_STATEMENT_DELETE ||
		this == ASSIGNMENT_DELETE ||
		this == BREAK_STATEMENT_DELETE ||
		this == CATCH_CLAUSE_DELETE ||
		this == CONSTRUCTOR_INVOCATION_DELETE ||
		this == DO_STATEMENT_DELETE ||
		this == FOR_STATEMENT_DELETE ||
		this == IF_STATEMENT_DELETE ||
		this == METHOD_INVOCATION_DELETE ||
		this == RETURN_STATEMENT_DELETE ||
		this == STATEMENT_DELETE ||
		this == SUPER_CONSTRUCTOR_INVOCATION_DELETE ||
		this == SUPER_METHOD_INVOCATION_DELETE ||
		this == SWITCH_CASE_DELETE ||
		this == SWITCH_STATEMENT_DELETE ||
		this == SYNCHRONIZED_STATEMENT_DELETE ||
		this == THROW_STATEMENT_DELETE ||
		this == TRY_STATEMENT_DELETE  ||
		this == VARIABLE_DECLARATION_STATEMENT_DELETE ||
		this == VARIABLE_INITIALIZE_DELETE || 
		this == WHILE_STATEMENT_DELETE; 
	}
	
	public static boolean isCommentBased (String op) {
		return valueOf(op).isCommentBased();
	}
	
	public boolean isCommentBased () {
		return 
			this == COMMENT_DELETE ||
			this == COMMENT_INSERT ||
			this == COMMENT_MOVE ||
			this == COMMENT_UPDATE ;
	}
}
