/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.reconciler;

import java.util.Set;

import org.eclipse.jdt.core.dom.AbstractTypeDeclaration;

import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.DiffCatParentMap;
import ca.mcgill.cs.swevo.diffcat.util.filter.IsClassDeleteFilter;
import ca.mcgill.cs.swevo.diffcat.util.filter.IsClassInsertFilter;
import ca.mcgill.cs.swevo.diffcat.util.filter.IsClassRenameFilter;
import ca.mcgill.cs.swevo.util.SetUtil;

/**
 * Eliminates detection of a class insert/delete when
 * we know the class insert/delete was already part of
 * a class rename operation. (Eliminates spurious results
 * but creates no non-essential diffs) 
 * 
 * @author dkawry
 */
public class ClassInsDelReconciler extends AbstractReconciler {

	protected DiffCatParentMap parentMap;

	@Override
	public Set<DiffCatResult> reconcile(Set<DiffCatResult> codeChanges) {
		
		Set<DiffCatResult> classInserts = SetUtil.filterAndCopy(codeChanges, new IsClassInsertFilter());
		Set<DiffCatResult> classDeletes = SetUtil.filterAndCopy(codeChanges, new IsClassDeleteFilter());
		Set<DiffCatResult> classRenames = SetUtil.filterAndCopy(codeChanges, new IsClassRenameFilter());
		
		for (DiffCatResult classInsert : classInserts) {
			AbstractTypeDeclaration inserted = (AbstractTypeDeclaration) classInsert.getNodeTo();
			for (DiffCatResult rename : classRenames) {
				if (rename.getRightParent() == inserted) {
					
					// ah, the class was renamed
					codeChanges.remove(classInsert);
				}
			}
		}
		
		for (DiffCatResult classDelete : classDeletes) {
			AbstractTypeDeclaration deleted = (AbstractTypeDeclaration) classDelete.getNodeFrom();
			for (DiffCatResult rename : classRenames) {
				if (rename.getLeftParent() == deleted) {
					
					// ah, the class was renamed
					codeChanges.remove(classDelete);
				}
			}
		}
		
		return codeChanges;
	}

}
