/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.reconciler;

import org.eclipse.jdt.core.dom.FieldDeclaration;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;
import ca.mcgill.cs.swevo.diffcat.util.filter.IsChangeOperationFilter;
import ca.mcgill.cs.swevo.util.filter.IFilter;

public class FieldRenameReconciler extends InsDelRenameReconciler {
	
	@Override
	protected SimpleName extractTo(DiffCatResult ins) {
		VariableDeclarationFragment insFrag = (VariableDeclarationFragment) ins.getNodeTo();
		SimpleName insName = insFrag.getName();
		return insName;
	}

	@Override
	protected SimpleName extractFrom(DiffCatResult del) {
		VariableDeclarationFragment delFrag = (VariableDeclarationFragment) del.getNodeFrom();
		SimpleName delName = delFrag.getName();
		return delName;
	}

	@Override
	protected IFilter<DiffCatResult> initDelFilter() {
		return new IsChangeOperationFilter(ChangeOperation.ATTRIBUTE_DELETE.name());
	}

	@Override
	protected IFilter<DiffCatResult> initInsFilter() {
		return new IsChangeOperationFilter(ChangeOperation.ATTRIBUTE_INSERT.name());
	}

	@Override
	protected boolean canCompare(DiffCatResult del, DiffCatResult ins) {
		if( del.getLeftParent() != ins.getLeftParent() ) {
			return false;
		}
		if ( del.getRightParent() != ins.getRightParent() ) {
			return false;
		}
		
		VariableDeclarationFragment delFrag = (VariableDeclarationFragment) del.getNodeFrom();
		VariableDeclarationFragment insFrag = (VariableDeclarationFragment) ins.getNodeTo();
		
		FieldDeclaration delDecl = (FieldDeclaration) delFrag.getParent();
		FieldDeclaration insDecl = (FieldDeclaration) insFrag.getParent();
		
		ITypeBinding delBinding = delDecl.getType().resolveBinding();
		ITypeBinding insBinding = insDecl.getType().resolveBinding();
		
		return BindingUtil.areEqual(delBinding, insBinding);
	}

	@Override
	protected ChangeOperation getRenameOp() {
		return ChangeOperation.ATTRIBUTE_NAME_UPDATE;
	}

	@Override
	protected boolean canInvestigate(DiffCatResult del, DiffCatResult ins, DiffCatResult change) {
		
		if (! change.isStatementUpdate()) {
			return false;
		}
		
		if (del.getLeftParent() != change.getLeftParent().getParent()) {
			return false;
		}
		
		if (del.getRightParent() != change.getRightParent().getParent()) {
			return false;
		}
		
		return true;
	}
}
