/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.reconciler;

import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SingleVariableDeclaration;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;
import ca.mcgill.cs.swevo.diffcat.util.filter.IsChangeOperationFilter;
import ca.mcgill.cs.swevo.util.filter.IFilter;

public class ParamRenameReconciler extends InsDelRenameReconciler {

	@Override
	protected IFilter<DiffCatResult> initDelFilter() {
		return new IsChangeOperationFilter (ChangeOperation.PARAMETER_DELETE.name());
	}

	@Override
	protected IFilter<DiffCatResult> initInsFilter() {
		return new IsChangeOperationFilter(ChangeOperation.PARAMETER_INSERT.name());
	}

	@Override
	protected boolean canCompare(DiffCatResult del, DiffCatResult ins) {
		
		if (del.getLeftParent() != ins.getLeftParent()) {
			return false;
		}
		
		if (del.getRightParent() != ins.getRightParent()) {
			return false;
		}
		
		ITypeBinding delBinding 
			= ((SingleVariableDeclaration) del.getNodeFrom()).getType().resolveBinding();
		
		ITypeBinding insBinding
			= ((SingleVariableDeclaration) ins.getNodeTo()).getType().resolveBinding();
		
		return (BindingUtil.areEqual(delBinding, insBinding));
	}

	@Override
	protected SimpleName extractFrom(DiffCatResult del) {
		return ((SingleVariableDeclaration) del.getNodeFrom()).getName();
	}

	@Override
	protected SimpleName extractTo(DiffCatResult ins) {
		return ((SingleVariableDeclaration) ins.getNodeTo()).getName();
	}

	@Override
	protected ChangeOperation getRenameOp() {
		return ChangeOperation.PARAMETER_NAME_UPDATE;
	}

	@Override
	protected boolean canInvestigate(DiffCatResult del, DiffCatResult ins, DiffCatResult change) {
		
		if (! change.isStatementUpdate()) {
			return false;
		}
		
		if (del.getLeftParent() != change.getLeftParent()) {
			return false;
		}
		
		if (del.getRightParent() != change.getRightParent()) {
			return false;
		}
		
		return true;
	}

}
