/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.reconciler;

import java.util.Arrays;
import java.util.Collection;
import java.util.Set;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.Type;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.DiffCatParentMap;
import ca.mcgill.cs.swevo.diffcat.util.filter.IsChangeOperationFilter;
import ca.mcgill.cs.swevo.util.CollectionUtil;
import ca.mcgill.cs.swevo.util.LRTuple;

public class ParentClassInsDelReconciler extends AbstractReconciler {

	private final IsChangeOperationFilter classInsDelFilter = 
		new IsChangeOperationFilter(
			ChangeOperation.PARENT_CLASS_DELETE.name(),
			ChangeOperation.PARENT_CLASS_INSERT.name()
		);
	
	private final IsChangeOperationFilter globalVarRenameFilter = 
		new IsChangeOperationFilter(
			ChangeOperation.CLASS_NAME_UPDATE.name()
		);
	
	private final RenameReconciler renameReconciler = new RenameReconciler();

	protected DiffCatParentMap parentMap;
	
	@Override
	public Set<DiffCatResult> reconcile(Set<DiffCatResult> input) {
		
		parentMap = new DiffCatParentMap(input);
		
		Collection<DiffCatResult> renames = CollectionUtil.filterAndCopy(parentMap.values(), globalVarRenameFilter);
		
		for (LRTuple<ASTNode> typePair : parentMap.getTypeKeys()) {
			Collection<DiffCatResult> results = parentMap.get(typePair);
			Collection<DiffCatResult> insDels = CollectionUtil.filterAndCopy(results, classInsDelFilter);
			
			if (insDels.isEmpty()) {
				continue;
			}
			
			if (insDels.isEmpty()) {
				continue;
			}
			
			for (LRTuple<DiffCatResult> pair : CollectionUtil.getUnorderedPairs(insDels))
			{
				DiffCatResult leftDiff = pair.getLeft();
				DiffCatResult rightDiff = pair.getRight();
				
				if (leftDiff.isDelete() == rightDiff.isDelete()) {
					continue;
				}
				
				Type leftType, rightType;
				String leftPath, rightPath;
				if (leftDiff.isDelete()) {
					leftType = (Type) leftDiff.getNodeFrom();
					rightType = (Type) rightDiff.getNodeTo();
					leftPath = leftDiff.getPathFrom();
					rightPath = rightDiff.getPathTo();
				}
				else 
				{
					rightType = (Type) leftDiff.getNodeTo();
					leftType = (Type) rightDiff.getNodeFrom();
					rightPath = leftDiff.getPathTo();
					leftPath = rightDiff.getPathFrom();
				}
				
				ITypeBinding leftBinding = leftType.resolveBinding();
				ITypeBinding rightBinding = rightType.resolveBinding();
				
				if (renameReconciler.onlyAffectedByRename(leftBinding, rightBinding, renames))
				{
					// found one (let the rename reconciler take care of the label)
					DiffCatResult constructorRename = new DiffCatResult();
					constructorRename.setChangeType(ChangeOperation.PARENT_CLASS_RENAME_UPDATE.name());
					constructorRename.setLeftParent(leftType.getParent());
					constructorRename.setRightParent(rightType.getParent());
					constructorRename.setNodeFrom(leftType);
					constructorRename.setNodeTo(rightType);
					constructorRename.setPathFrom(leftPath);
					constructorRename.setPathTo(rightPath);
					
					// remove the ins del pair
					parentMap.removeAll(Arrays.asList(leftDiff, rightDiff));
					
					// add ourselves
					parentMap.get(typePair).add(constructorRename);
				}
			}
		}
		
		return parentMap.values();
	}

}
