/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.refiner;

import java.util.List;
import java.util.Set;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.CastExpression;
import org.eclipse.jdt.core.dom.Expression;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.InfixExpression;
import org.eclipse.jdt.core.dom.PostfixExpression;
import org.eclipse.jdt.core.dom.PrefixExpression;
import org.eclipse.jdt.core.dom.Type;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;

public class CastExpressionRefiner extends AbstractRefiner {

	// haven't gotten this to work anywhere, since lack of bindings 
	// makes it impossible to check in all samples (ant 2132, 2145, 2154)
	// found one here: ant 2146, nutch 103
	
	@Override
	public Set<DiffCatResult> refine(DiffCatResult result) {
		
		// TODO, xerces 2229 
		if (! result.isStatementUpdate() || result.getNodeFrom()==null || result.getNodeTo() == null) {
			return singleton(result);
		}
		
		if (sameModuloRedundantCastExpressions(result.getNodeFrom(), result.getNodeTo())) {
			result.setChangeType(ChangeOperation.REDUNDANT_CLASS_CAST_UPDATE.name());
		}
		
		return singleton(result);
	}

	private boolean sameModuloRedundantCastExpressions (ASTNode one, ASTNode two)
	{
		boolean atLeastOneCastExpression = false;
		
		List<ASTNode> oneFlattened = flatten(one);
		List<ASTNode> twoFlattened = flatten(two);
		
		int oneCounter = 0;
		int twoCounter = 0;
		while(oneCounter < oneFlattened.size() && twoCounter < twoFlattened.size()){
			ASTNode subOne = oneFlattened.get(oneCounter);
			ASTNode subTwo = twoFlattened.get(twoCounter);
			
			if (subOne.getNodeType() == subTwo.getNodeType()) {
				if (! areSame(subOne, subTwo)) {
					return false;
				}
				
				oneCounter ++;
				twoCounter ++;
			}
			else if (subOne.getNodeType() == ASTNode.CAST_EXPRESSION) {
				CastExpression ce = (CastExpression) subOne;
				if (! isRedundant (ce)) {
					return false;
				}
				Type type = ce.getType();
				if (type.isPrimitiveType()) {
					oneCounter += 2;
				} else {
					oneCounter += 3;
				}
				atLeastOneCastExpression = true;
			}
			else if (subTwo.getNodeType() == ASTNode.CAST_EXPRESSION) {
				CastExpression ce = (CastExpression) subTwo;
				if (! isRedundant (ce)) {
					return false;
				}
				Type type = ce.getType();
				if (type.isPrimitiveType()) {
					twoCounter += 2;
				} else {
					twoCounter += 3;
				}
				atLeastOneCastExpression = true;
			}
			else {
				return false;
			}
		}
		
		if (! atLeastOneCastExpression ) {
			return false;
		}
		
		return oneCounter == oneFlattened.size() && twoCounter == twoFlattened.size();
	}

	private boolean isRedundant (CastExpression ce) {
		Type type = ce.getType();
		ITypeBinding castBinding = type.resolveBinding();
		
		Expression expression = ce.getExpression();
		ITypeBinding exprBinding = expression.resolveTypeBinding();
		
		return BindingUtil.areEqual(castBinding, exprBinding);
	}
}
