package ca.mcgill.cs.swevo.diffcat.refiner;

import java.util.Collections;
import java.util.Set;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.BooleanLiteral;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.NullLiteral;
import org.eclipse.jdt.core.dom.NumberLiteral;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOp;
import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.filter.IsChangeOperationFilter;
import ca.mcgill.cs.swevo.util.filter.IFilter;

public class InitRefiner extends AbstractRefiner {

	private final IFilter<DiffCatResult> isInit 
	 = new IsChangeOperationFilter(
			 ChangeOperation.VARIABLE_INITIALIZE_DELETE.name(),
			 ChangeOperation.VARIABLE_INITIALIZE_INSERT.name(),
			 ChangeOperation.ATTRIBUTE_INIT_DELETE.name(), 
			 ChangeOperation.ATTRIBUTE_INIT_INSERT.name()); 
	
	@Override
	public Set<DiffCatResult> refine(DiffCatResult result) {
		
		if (! isInit.passesFilter(result)) {
			return Collections.singleton(result);
		}
		
		ASTNode init = result.isDelete() ? result.getNodeFrom() : result.getNodeTo();
		VariableDeclarationFragment frag = (VariableDeclarationFragment) init.getParent();
		
		if (isRedundantInit(init, frag)) {
			result.setChangeType(mapChangeType(result.getChangeType()));
		}
		
		return Collections.singleton(result);
	}

	private String mapChangeType (String changeType) {
		return "TRIVIAL_" + changeType;
	}
	
	private boolean isRedundantInit (ASTNode init, VariableDeclarationFragment frag)
	{
		ITypeBinding binding = frag.resolveBinding().getType();
		String type = binding.getQualifiedName();
		if (type.equals("int")) {
			if (init.getNodeType() == ASTNode.NUMBER_LITERAL) {
				NumberLiteral lit = (NumberLiteral) init;
				String token = lit.getToken();
				return token.equals("0");
			}
		} else if (type.equals("double")) {
			if (init.getNodeType() == ASTNode.NUMBER_LITERAL) {
				NumberLiteral lit = (NumberLiteral) init;
				String token = lit.getToken();
				return token.equals("0") || token.equals("0.0") || 
				       token.equals("0.0f") || token.equals("0f"); 
			}
		} else if (type.equals("float")) {
			if (init.getNodeType() == ASTNode.NUMBER_LITERAL) {
				NumberLiteral lit = (NumberLiteral) init;
				String token = lit.getToken();
				return token.equals("0") ||  
				       token.equals("0.0f") || token.equals("0f"); 
			}
		}
		else if (type.equals("boolean")) {
			if (init.getNodeType() == ASTNode.BOOLEAN_LITERAL) {
				BooleanLiteral lit = (BooleanLiteral) init;
				String token = lit.toString();
				return token.equals("false");
			}
			
		} else if (! binding.isPrimitive()) {
			if (init.getNodeType() == ASTNode.NULL_LITERAL) {
				return true;
			}
		} 
		
		return false;
	}
}
