/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.util;

import java.util.Collection;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

import org.eclipse.jdt.core.dom.ASTNode;

import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.filter.MethodNodeFilter;
import ca.mcgill.cs.swevo.diffcat.util.filter.TypeNodeFilter;
import ca.mcgill.cs.swevo.util.LRTuple;
import ca.mcgill.cs.swevo.util.LRTupleSet;

public class DiffCatParentMap { 

	private Map<LRTuple<ASTNode>, Collection<DiffCatResult>> parentMap;
	
	public DiffCatParentMap () {
		parentMap = new HashMap<LRTuple<ASTNode>, Collection<DiffCatResult>>();
	}
	
	public DiffCatParentMap (Collection<DiffCatResult> diffCatResults) {
		this();
		initPairs (diffCatResults);
	}
	
	private void initPairs (Collection<DiffCatResult> diffCatResults) {
		for (DiffCatResult result : diffCatResults) {
			addPair (result);
		}
	}
	
	public void clear () {
		parentMap.clear();
	}
	
	public void addPair (DiffCatResult result) {
		ASTNode leftParent = result.getLeftParent();
		ASTNode rightParent = result.getRightParent(); 
		
		LRTuple<ASTNode> parents = null;
		parents = new LRTuple<ASTNode>(leftParent, rightParent);
		if (parentMap.get(parents) == null) {
			parentMap.put(parents, new LinkedList<DiffCatResult>());
		}
		parentMap.get(parents).add(result);
	}
	
	public void addPairs (Collection<DiffCatResult> results) {
		for (DiffCatResult result : results) {
			addPair (result);
		}
	}
	
	public LRTupleSet<ASTNode> keySet () {
		return new LRTupleSet<ASTNode>(parentMap.keySet());
	}
	
	public LRTupleSet<ASTNode> getMethodKeys () {
		LRTupleSet<ASTNode> keySet = keySet();
		keySet.retainAll(new MethodNodeFilter());
		return keySet; 
	}
	
	public LRTupleSet<ASTNode> getTypeKeys () {
		LRTupleSet<ASTNode> keySet = keySet();
		keySet.retainAll(new TypeNodeFilter());
		return keySet;
	}
	
	public Collection<DiffCatResult> get (LRTuple<ASTNode> methodPair) {
		return parentMap.get(methodPair);
	}
	
	public void set (LRTuple<ASTNode> pair, Collection<DiffCatResult> results) {
		parentMap.put(pair, results);
	}
	
	public Set<DiffCatResult> values () {
		Set<DiffCatResult> vals = new HashSet<DiffCatResult>();
		for (Collection<DiffCatResult> val : parentMap.values()) {
			vals.addAll(val);
		}
		return vals;
	}
	
	public void removeAll (Collection<DiffCatResult> results) {
		for (LRTuple<ASTNode> p : parentMap.keySet()) {
			Collection<DiffCatResult> collection = parentMap.get(p);
			collection.removeAll(results);
		}
	}
}
