/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.util;

import java.util.Set;

import org.eclipse.jdt.core.dom.ASTNode;
import org.eclipse.jdt.core.dom.SimpleName;

import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.ast.NameChanger;

public class RenameHelper {

	public void undoRenames (ASTNode node, Set<DiffCatResult> renames) {
		for (DiffCatResult rename : renames) {
			undoRename (node, rename);
		}
	}
	
	public void redoRenames (ASTNode node, Set<DiffCatResult> renames) {
		for (DiffCatResult rename : renames) {
			redoRename (node, rename);
		}
	}
	
	private void undoRename (ASTNode node, DiffCatResult rename) {
		SimpleName left = (SimpleName) rename.getNodeFrom();
		SimpleName right = (SimpleName) rename.getNodeTo();
		
		doRename (node, right, left);
	}
	
	private void redoRename (ASTNode node, DiffCatResult rename) {
		SimpleName left = (SimpleName) rename.getNodeTo();
		String right = rename.getHandleTo();
		
		doRename (node, left, right);
	}
	
	private void doRename (ASTNode node, SimpleName from, SimpleName to) 
	{ 
		node.accept(new NameChanger(from, to));
	}
	
	private void doRename (ASTNode node, SimpleName from, String to)
	{
		node.accept(new NameChanger(from, to));
	}
	
}
