/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.util.ast;

import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.FieldAccess;
import org.eclipse.jdt.core.dom.IBinding;
import org.eclipse.jdt.core.dom.IVariableBinding;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SuperFieldAccess;

import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;

/**
 * Collects all references to a field within an ASTNode.
 * @author dkawry
 */

public class FieldAccessCollector extends ASTVisitor {

	private List<IVariableBinding> fieldAccesses;
	
	public FieldAccessCollector () {
		this.fieldAccesses = new LinkedList<IVariableBinding>();
	}
	
	public void clear () {
		this.fieldAccesses.clear();
	}
	
	public List<IVariableBinding> getFieldAccesses () {
		return Collections.unmodifiableList(fieldAccesses);
	}
	
	public void endVisit (FieldAccess access) {
		IVariableBinding binding = access.resolveFieldBinding();
		
		if (BindingUtil.validateBinding(binding)) {
			fieldAccesses.add (binding);
		}
	}
	
	public void endVisit (SuperFieldAccess access) {
		IVariableBinding binding = access.resolveFieldBinding();
		
		if (BindingUtil.validateBinding(binding)) {
			fieldAccesses.add (binding);
		}
	}
	
	public void endVisit (SimpleName name) {
		IBinding binding = name.resolveBinding();
		
		if (binding == null) {
			return;
		}
		
		if (binding.getKind() != IBinding.VARIABLE) {
			return;
		}
		
		IVariableBinding varBinding = (IVariableBinding) binding;
		
		if (varBinding.isField() && BindingUtil.validateBinding(varBinding)) {
			fieldAccesses.add(varBinding);
		}
	}
}
