/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.util.ast;

import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.List;
import java.util.Set;

import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.ConstructorInvocation;
import org.eclipse.jdt.core.dom.IBinding;
import org.eclipse.jdt.core.dom.IMethodBinding;
import org.eclipse.jdt.core.dom.MethodInvocation;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.SuperConstructorInvocation;
import org.eclipse.jdt.core.dom.SuperMethodInvocation;

import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;

public class MethodInvocationCollector extends ASTVisitor {

	private List<IMethodBinding> methodCalls; 
	
	public MethodInvocationCollector () {
		methodCalls = new LinkedList<IMethodBinding>();
	}
	
	public List<IMethodBinding> getMethodCalls () {
		return Collections.unmodifiableList(methodCalls);
	}
	
	public void clearMethodCalls () {
		methodCalls.clear();
	}
	
	public void endVisit (MethodInvocation invocation) {
		methodCalls.add( invocation.resolveMethodBinding() );
	}
	
	public void endVisit (SuperMethodInvocation invocation) {
		methodCalls.add( invocation.resolveMethodBinding());
	}
	
	public void endVisit (ConstructorInvocation invocation) {
		methodCalls.add(invocation.resolveConstructorBinding());
	}
	
	public void endVisit (SuperConstructorInvocation invocation) {
		methodCalls.add(invocation.resolveConstructorBinding());
	}
}
