/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.util.ast;

import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.IBinding;
import org.eclipse.jdt.core.dom.IMethodBinding;
import org.eclipse.jdt.core.dom.ITypeBinding;
import org.eclipse.jdt.core.dom.SimpleName;

import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;


public class NameChanger extends ASTVisitor {

	private final IBinding from;
	private final String to;
	
	public NameChanger (SimpleName from, SimpleName to) {
		this.from = from.resolveBinding();
		this.to = to.toString();
	}
	
	public NameChanger(SimpleName from2, String to2) {
		this.from = from2.resolveBinding();
		this.to = to2;
	}

	public boolean visit (SimpleName name) {
		
		IBinding binding = name.resolveBinding();
		if (BindingUtil.areEqualBindings(binding, from)) {
			name.setIdentifier(to);
		}
		else if (from instanceof ITypeBinding && binding instanceof IMethodBinding) {
			IMethodBinding nameBinding = (IMethodBinding) binding;
			
			// if referring to constructor of renamed class, do the rename
			if (nameBinding.isConstructor() && BindingUtil.areEqualBindings(from, nameBinding.getDeclaringClass())) {
				name.setIdentifier(to);
			}
		}
		
		return true;
	}
}
