/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.util.ast;

import java.util.ArrayList;
import java.util.Collections;
import java.util.LinkedList;
import java.util.List;

import org.eclipse.jdt.core.dom.ASTVisitor;
import org.eclipse.jdt.core.dom.SimpleName;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;

import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;

/**
 * Collects uses of a simple name in an ast.
 * NOTE: uses == comparison for bindings, so cannot be used for
 *  cross version comparison
 * @author dkawry
 *
 */

public class VariableUsageCollector extends ASTVisitor {

	private final SimpleName variable;
	
	private final List<SimpleName> occurrences;
	
	public VariableUsageCollector (SimpleName variable) {
		this.variable = variable;
		this.occurrences = new LinkedList<SimpleName>();
	}
	
	public boolean visit (VariableDeclarationFragment fragment) {
		
		if (fragment.getName() == variable) {
			return false;
		}
		
		return true;
	}
	
	public void endVisit (SimpleName name) {
		
		if (name.resolveBinding() != variable.resolveBinding()) {
			return;
		}
		
		// found an occurrence 
		occurrences.add(name);
	}
	
	public void clear () {
		occurrences.clear();
	}
	
	public List<SimpleName> getOccurrences () {
		return new ArrayList<SimpleName>(occurrences);
	}
}
