/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.diffcat.util.op;

import org.eclipse.jdt.core.dom.AbstractTypeDeclaration;
import org.eclipse.jdt.core.dom.IBinding;
import org.eclipse.jdt.core.dom.IVariableBinding;
import org.eclipse.jdt.core.dom.MethodDeclaration;
import org.eclipse.jdt.core.dom.VariableDeclarationFragment;

import ca.mcgill.cs.swevo.diffcat.model.ChangeOperation;
import ca.mcgill.cs.swevo.diffcat.model.DiffCatResult;
import ca.mcgill.cs.swevo.diffcat.util.binding.BindingUtil;
import ca.mcgill.cs.swevo.util.op.IOperation;

public class SignatureReconcileOp implements IOperation<DiffCatResult> {

	@Override
	public DiffCatResult apply(DiffCatResult t) {
		String changeType = t.getChangeType();
		if (changeType.equals(ChangeOperation.FUNCTIONALITY_DELETE.name())) {
			MethodDeclaration decl = (MethodDeclaration) t.getNodeFrom();
			t.setLeftMethodSignature(BindingUtil.toLongHand(decl.resolveBinding()));
		}
		else if (changeType.equals(ChangeOperation.TRIVIAL_CONSTRUCTOR_DELETE.name())) {
			MethodDeclaration decl = (MethodDeclaration) t.getNodeFrom();
			t.setLeftMethodSignature(BindingUtil.toLongHand(decl.resolveBinding()));
		}
		else if (changeType.equals(ChangeOperation.FUNCTIONALITY_INSERT.name())) {
			MethodDeclaration decl = (MethodDeclaration) t.getNodeTo();
			t.setRightMethodSignature(BindingUtil.toLongHand(decl.resolveBinding()));
		}
		else if (changeType.equals(ChangeOperation.TRIVIAL_CONSTRUCTOR_INSERT.name())) {
			MethodDeclaration decl = (MethodDeclaration) t.getNodeTo();
			t.setRightMethodSignature(BindingUtil.toLongHand(decl.resolveBinding()));
		}
		else if (changeType.equals(ChangeOperation.CLASS_DELETE.name())) {
			AbstractTypeDeclaration decl = (AbstractTypeDeclaration) t.getNodeFrom();
			t.setLeftClassSignature(BindingUtil.toLongHand(decl.resolveBinding()));
		}
		else if (changeType.equals(ChangeOperation.CLASS_INSERT.name())) {
			AbstractTypeDeclaration decl = (AbstractTypeDeclaration) t.getNodeTo();
			t.setRightClassSignature(BindingUtil.toLongHand(decl.resolveBinding()));
		}
		else if (changeType.equals(ChangeOperation.ATTRIBUTE_DELETE.name())) {
			VariableDeclarationFragment frag = (VariableDeclarationFragment) t.getNodeFrom();
			IVariableBinding resolveBinding = frag.resolveBinding();
			if (resolveBinding != null) {
				// TODO xerces ppa bug 514, ant 4236
				t.setLeftFieldSignature(BindingUtil.toLongHand(resolveBinding));
			} else {
				t.setLeftFieldSignature("");
			}
		}
		else if (changeType.equals(ChangeOperation.ATTRIBUTE_INSERT.name())) {
			VariableDeclarationFragment frag = (VariableDeclarationFragment) t.getNodeTo();
			IVariableBinding binding = frag.resolveBinding();
			if (binding != null) {
				t.setRightFieldSignature(BindingUtil.toLongHand(binding));
			} else {
				t.setRightFieldSignature("");
			}
		}
		
		return t;
	}

}
