package ca.mcgill.cs.swevo.util;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import java.util.Set;
import java.util.Stack;

import ca.mcgill.cs.swevo.util.op.IConverter;

public class DirectedGraphUtil {

	public static <T> Map<T, Set<T>> buildTransitiveClosure (Collection<T> nodes, IConverter<T, Collection<T>> c)
	{
		Map<T, Set<T>> graph = new HashMap<T, Set<T>>();
		Queue<T> queue = new LinkedList<T>();
		
		for (T node : nodes) {
			queue.add(node);
			
			while (! queue.isEmpty())
			{
				node = queue.poll();
				
				if (graph.containsKey(node)) {
					continue;
				}
				
				graph.put(node, new HashSet<T>());
				graph.get(node).add(node);
				
				Collection<T> links = c.convert(node);
				
				Set<T> expansion = new HashSet<T>(); 
				for (T link : links) 
				{
					if (graph.containsKey(link)) {
						expansion.addAll(graph.get(link));
					} else {
						expansion.add(link);
					}
				}
				
				graph.get(node).addAll(expansion);
				
				for (T key : graph.keySet()) {
					if (graph.get(key).contains(node)) {
						graph.get(key).addAll(expansion);
					}
				}
				
				queue.addAll(links);
			}
		}
			
		return graph;
	}
	
}
