/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.util;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;

import ca.mcgill.cs.swevo.util.filter.IFilter;
import ca.mcgill.cs.swevo.util.op.IConverter;

public class ListUtil {

	/**
	 * Returns the index of the first element that survives the filter. 
	 */
	public static <T> int firstIndexOf (List<? extends T> list, IFilter<T> filter) {
		for (int i = 0; i < list.size(); i++) {
			if (filter.passesFilter(list.get(i))) {
				return i;
			}
		}
		return -1;
	}

	public static <T> List<T> filterAndCopy (List<? extends T> list, IFilter<T> filter) {
		List<T> newList = new ArrayList<T>(list.size());
		for (T element : list) {
			if (filter.passesFilter(element)) {
				newList.add(element);
			}
		}
		return newList;
	}
	
	public static <T> List<T> filterAndCopy (List<T> list, Collection<IFilter<T>> filters) {
		List<T> retain = new ArrayList<T>(list.size());
		for (T item : list) {
			boolean passes = true;
			for (IFilter<T> filter : filters)
				passes &= passes? filter.passesFilter(item) : false;
			if (passes) retain.add(item);
		}
		return retain;
	}
	
	public static <F, T> List<T> convert (List<F> col, IConverter<F, T> converter) {
		List<T> newCol = new ArrayList<T>(col.size());
		for (F el : col) {
			newCol.add(converter.convert(el));
		}
		return newCol;
	}
}
