package ca.mcgill.cs.swevo.util;

import java.util.Collection;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Set;

public class UndirectedGraph <T>  {

	private final Map<LRTuple<T>, String> edges = new HashMap<LRTuple<T>, String>();
	private final CollectionMap<T, T> graph = new CollectionMap<T, T>();
	
	public UndirectedGraph<T> getComplementGraph ()
	{
		UndirectedGraph<T> complement = new UndirectedGraph<T>();
		for (T node : graph.keySet()) {
			complement.initNode(node);
		}
		for (LRTuple<T> node : graph.keyPairSet()) {
			if (! hasEdge(node)) {
				complement.addEdge(node.getLeft(), node.getRight());
			}
		}
		
		return complement;
	}
	
	public UndirectedGraph<T> copy () {
		UndirectedGraph<T> copy = new UndirectedGraph<T>();
		for (T node : graph.keySet()) {
			copy.initNode(node);
			for (T neigh : graph.get(node)) {
				copy.addEdge(node, neigh);
			}
		}
		return copy;
	}
	
	public void initNode (T node) {
		graph.get(node);
	}
	
	public Collection<T> getEdges (T node) {
		return graph.get(node);
	}
	
	public Collection<LRTuple<T>> getAllEdges () {
		Collection<LRTuple<T>> allEdges = new LinkedList<LRTuple<T>>();
		for (T node1 : graph.keySet()) {
			for (T node2 : graph.get(node1)) {
				LRTuple<T> t = new LRTuple<T>(node2, node1);
				if (allEdges.contains(t)) {
					continue;
				}
				t.setLeft(node1);
				t.setRight(node2);
				if (allEdges.contains(t)) {
					continue;	// ? 
				}
				allEdges.add(t);
			}
		}
		return allEdges;
	}
	
	public String getEdgeValue (LRTuple<T> edge) {
		return edges.get(edge);
	}
	
	public String getEdgeValue (T v1, T v2) {
		return getEdgeValue (new LRTuple<T>(v1, v2));
	}
	
	public boolean hasEdges (T node) {
		return ! graph.get(node).isEmpty();
	}
	
	public boolean hasNode (T node) {
		return graph.containsKey(node);
	}
	
	public void addEdge (LRTuple<T> pair) {
		addEdge (pair.getLeft(), pair.getRight());
	}
	
	public void addEdge (T node1, T node2) {
		addEdge (node1, node2, "");
	}
	
	public void addEdge (T node1, T node2, String edgeVal) {
		if (! graph.get(node1).contains(node2)) {
			graph.add(node1, node2);
		}
		if (! graph.get(node2).contains(node1)) {
			graph.add(node2, node1);
		}
		edges.put(new LRTuple<T>(node1, node2), edgeVal);
		edges.put(new LRTuple<T>(node2, node1), edgeVal);
	}
	
	public void removeEdge (T node1, T node2) {
		if (! graph.containsKey(node1)) {
			return;
		}
		
		graph.get(node1).remove(node2);
		graph.get(node2).remove(node1);
		edges.remove(new LRTuple<T>(node1, node2));
		edges.remove(new LRTuple<T>(node2, node1));
	}
	
	public void addAllEdges (T node1, Collection<T> nodes) {
		for (T node : nodes) {
			addEdge(node1, node);
		}
	}
	
	void addEdges (T...nodes) {
		for (int i = 0; i < nodes.length; i++) {
			for (int j = i+1; j < nodes.length; j++) {
				addEdge (nodes[i], nodes[j]);
			}
		}
	}
	
	public Set<T> getNodes () {
		return graph.keySet();
	}
	
	public boolean hasEdge (LRTuple<T> nodePair) {
		return hasEdge (nodePair.getLeft(), nodePair.getRight());
	}
	
	public boolean hasEdge (T node1, T node2) {
		if (! graph.containsKey(node1)) {
			return false;
		}
		
		return graph.get(node1).contains(node2);
	}
	
	public void clear () {
		graph.clear();
		edges.clear();
	}
}
