package ca.mcgill.cs.swevo.util;

import java.util.Collection;
import java.util.Collections;
import java.util.HashMap;
import java.util.HashSet;
import java.util.LinkedList;
import java.util.Map;
import java.util.Queue;
import java.util.Set;
import java.util.TreeSet;

import ca.mcgill.cs.swevo.util.op.IConverter;

public class UndirectedGraphUtil {

	public static <T> Collection<UndirectedGraph<T>> getConnectedSubgraphs (UndirectedGraph<T> graph)
	{
		Set<T> nodesUnseen = new HashSet<T>(graph.getNodes());
		Collection<UndirectedGraph<T>> graphs = new LinkedList<UndirectedGraph<T>>();
		
		Queue<T> toProcess = new LinkedList<T>();
		while (! nodesUnseen.isEmpty())
		{
			T node = CollectionUtil.getFirst(nodesUnseen);
			toProcess.add(node);
			nodesUnseen.remove(node);
			node = null;
			
			UndirectedGraph<T> newGraph = new UndirectedGraph<T>();
			
			while (! toProcess.isEmpty()) {
				T poll = toProcess.poll();
				Collection<T> edges = graph.getEdges(poll);
				
				newGraph.initNode(poll);
				newGraph.addAllEdges(poll, edges);
				for (T edge : edges) {
					if (nodesUnseen.contains(edge)) {
						toProcess.add(edge);
						nodesUnseen.remove(edge);
					}
				}
			}
			
			graphs.add(newGraph);
		}
		
		return graphs;
	}
	
	public static <T> Set<Set<T>> getMaximalKnSubgraphs2 (UndirectedGraph<T> graph)
	{
		Collection<UndirectedGraph<T>> connGraphs = getConnectedSubgraphs(graph);
		
		Set<Set<T>> results = new HashSet<Set<T>>();
		
		for (UndirectedGraph<T> graff : connGraphs) {
			// assume connected, for now
			Set<T> nodes = new HashSet<T>(graff.getNodes());
			
			Queue<Set<T>> toProcess = new LinkedList<Set<T>>();
			toProcess.add(nodes);
			
			while (! toProcess.isEmpty()) {
				Set<T> poll = toProcess.poll();
				if (isClique(poll, graff)) {
					results.add(poll);
				}
				else {
					// find non-edge  
					LRTuple<T> nonEdge = null;
					for (LRTuple<T> nodePair : CollectionUtil.getUnorderedPairs(poll)) {
						if (nonEdge == null) {
							if (! graff.hasEdge(nodePair)) {
								nonEdge = nodePair;
							}
						}
					}
					
					// split on nonedge
					Set<T> left = new HashSet<T>(poll);
					left.remove(nonEdge.getRight());
					
					Set<T> right = new HashSet<T>(poll);
					right.remove(nonEdge.getLeft());
					
					// only add non-singletons 
					// since we assumed everyone had at least one edge in the original split
					if (left.size() == 2) {
						T f = CollectionUtil.getNth(left, 1);
						T s = CollectionUtil.getNth(left, 2);
						
						if (graph.hasEdge(f, s)) {
							toProcess.add(left);
						}
					}
					else {
						toProcess.add(left);
					}
					
					if (right.size() == 2) {
						T f = CollectionUtil.getNth(right, 1);
						T s = CollectionUtil.getNth(right, 2);
						
						if (graph.hasEdge(f, s)) {
							toProcess.add(right);
						}
					}
					else {
						toProcess.add(right);
					}
					
					
				}
			}
		}
			
		return results;
	}
	
	public static <T> boolean isClique (Set<T> nodes, UndirectedGraph<T> graph)
	{
		boolean isClique = true;
		for (T node : nodes) {
			if (! isClique) {
				continue;
			}
			Collection<T> edges = graph.getEdges(node);
			edges.add(node);
			if (! edges.containsAll(nodes)) {
				isClique = false;
			}
			edges.remove(node);
		}
		
		return isClique;
	}
}
