/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.util.io;

import java.io.File;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Scanner;

public class IOUtils {

	public static void closeQuietly (Scanner scanner) {
		try {
			scanner.close();
		} catch (Exception e) { }
	}
	
	public static void closeQuietly (PrintWriter pw) {
		try {
			pw.close();
		} catch (Exception e) { }
	}
	
	public static List<File> openAllFiles (List<String> filePaths) 
	{
		List<File> files = new LinkedList<File>();
		for (String fp : filePaths) {
			files.add(new File(fp));
		}
		return files;
	}
	
	public static List<String> readLinesUpTo (Scanner scanner, String toMatch) {
		
		// to indicate error
		if (! scanner.hasNextLine()) {
			return null;
		}
		
		String lastRead = null;
		ArrayList<String> read = new ArrayList<String>();
		do {
			lastRead = scanner.nextLine();
			if (lastRead.equals(toMatch)) {
				return read;
			}
			else {
				read.add(lastRead);
			}
			
		} while (scanner.hasNextLine());
		
		// to indicate error
		return null;
	}
}
