/*******************************************************************************
 * Copyright (c) 2011 David Kawrykow.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *     David Kawrykow - initial API and implementation
 ******************************************************************************/
package ca.mcgill.cs.swevo.util;

import static org.junit.Assert.*;

import org.junit.Test;

public class TestStringUtil {

	@Test
	public void testEliminateDocs ()
	{
		String function = "/** Some Java doc */\n public /* haha */ void f(){}";
		function = StringUtil.eliminateDocs(function);
		
		assertEquals("\n public  void f(){}", function);
	}
	
	@Test 
	public void testEliminateComments ()
	{
		String function = "void f( ) {\n\\comment\nnocomment\\comment!\n}";
		function = StringUtil.eliminateComments(function);
		
		assertEquals("void f( ) {\n\\comment\nnocomment\\comment!\n}", function);
	}
	
	@Test
	public void testFindMethodSignature ()
	{
		String function = "void f( ){\n\\comment\nnocomment\\comment!\n}";
		function = StringUtil.findMethodSignature(function);
		
		assertEquals("void f( )", function);
	}
	
	@Test
	public void testFindFieldSignature () 
	{
		String field = "Object f   = something;";
		field = StringUtil.findFieldSignature(field);
		assertEquals ("Object f", field);
		
		field = "Object f  ;";
		field = StringUtil.findFieldSignature(field);
		assertEquals ("Object f", field);
	}
	
	@Test
	public void testNormalizeWhiteSpace () 
	{
		String function = "void  f(\t    )";
		function = StringUtil.normalizeWhitespace(function);
		
		assertEquals("void f( )", function);
	}
	
	@Test 
	public void testEliminateTypeArgs ()
	{
		String function = "void Map<Hello,Hoho<?>> f(Collection<HOLLO> p)";
		function = StringUtil.eliminateTypeArgs(function);
		
		assertEquals("void Map f(Collection p)", function);
	}
	
	@Test
	public void testNormalizeType ()
	{
		String type = "java.lang.String";
		type = StringUtil.normalizeType(type);
		
		assertEquals ("String", type);
		
		type = "java.lang.String.InnerType";
		type = StringUtil.normalizeType(type);
		
		assertEquals ("String.InnerType", type);
		
		type = "String";
		type = StringUtil.normalizeType(type);
		
		assertEquals ("String", type);
	}
	
	@Test
	public void testMethodSigShortHand ()
	{
		String function = "/** some java doc \n * hahaha \n /*\n@Test public void f /* oops! */ (String s, String y)";
		function += "\n{\n\t body statements;\n body statements;\n}\n  ";
		
		String shorthand = StringUtil.getShortHandMethodSig(function);
		
		assertEquals ("f(String,String)", shorthand);
	}
	
	@Test
	public void testFieldSigShortHand () 
	{
		String field = "/** Some field \n */  public java.lang.String   f   ;\n\n";
		
		String shorthand = StringUtil.getShortHandFieldSig(field);
		
		assertEquals ("f", shorthand);
	}
	
	@Test
	public void testRandomFunction01 ()
	{
		String function = "/** bla */\n public void name(AntTypeDefinition p){\nblablabla\n}";
		String shorthand = StringUtil.getShortHandMethodSig(function);
		
		assertEquals ("name(AntTypeDefinition)", shorthand);
	}
	
	@Test 
	public void testRandomField01 ()
	{
		String field = "m_timestamp=null";
		String shorthand = StringUtil.getShortHandFieldSig(field);
		
		assertEquals ("m_timestamp", shorthand);
	}
}
